# This work is based upon CMFExternalFile, which was developed by
# Alan Runyan with contributions by Andy McKay, Kiran Jonnalagadda
# and PloneExternalFile developed by Florian Geier.
#
# CMFManagedFile extends CMFExternalFile to include a concept that
# can be found in PloneExternalFile where uploaded documents may be
# routed to specific directories or file systems.  This is specified by the
# user at upload time through the selection of pre-defined "repositories"
# that the site administrator would define and establish.
#
# The merger of CMFExternalFile and PloneExternalFile has been further
# extended to include some aspects of managing the external files on the
# file systems.  These include the automatic deletion of the file system files
# for deleted documents of the class CMFManagedFile governed by customizable
# and user selectable deletion policies, and the automatic movement of files
# from one file system repository to another based upon an edit action
# that includes the selection of a repository other than the repository
# in which the file is currently stored.
#
"""
$Id: test_basic.py 483 2004-11-24 17:29:38Z sidnei $
"""

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

# Load fixture
import unittest
import tempfile
from base import doctest, FunctionalTest

from Products.CMFManagedFile.config import *
from Products.ATManagedFile.config import *

class BasicOperationsTest(FunctionalTest):

    def setUp(self):
        FunctionalTest.setUp(self)
        self.tmp_dir = tempfile.mkdtemp()
        self.sub_dir = os.path.join(self.tmp_dir, 'sub')
        os.mkdir(self.sub_dir)
        self.test_file = tempfile.NamedTemporaryFile(dir=self.tmp_dir)
        self.cleanups.append(self.test_file.name)
        self.cleanup_dirs.append(self.tmp_dir)

    def tearDown(self):
        try:
            self.test_file.close()
        except OSError:
            pass
        FunctionalTest.tearDown(self)

class UseCaseTest(FunctionalTest):

    path_policy = None
    filename_policy = ORIGINAL_POLICY

    def afterSetUp(self):
        FunctionalTest.afterSetUp(self)
        tmp_dir = tempfile.mkdtemp()
        self.cleanup_dirs.append(tmp_dir)
        # Set the default repository path to point to the tmp_dir:
        repo = self.tool.getRepository('default')
        repo.edit(path=tmp_dir, path_policy=self.path_policy)

    def tearDown(self):
        FunctionalTest.tearDown(self)

class UIDOriginalPolicyUseCase(UseCaseTest):
    path_policy = FS_PHYSICAL_UID

class RelativePathOriginalPolicyUseCase(UseCaseTest):
    path_policy = FS_RELATIVE

class PhysicalPathOriginalPolicyUseCase(UseCaseTest):
    path_policy = FS_PHYSICAL

class BasePathOriginalPolicyUseCase(UseCaseTest):
    path_policy = FS_BASE

class UIDGeneratedPolicyUseCase(UseCaseTest):
    path_policy = FS_PHYSICAL_UID
    filename_policy = GENERATE_POLICY

class RelativePathGeneratedPolicyUseCase(UseCaseTest):
    path_policy = FS_RELATIVE
    filename_policy = GENERATE_POLICY

class PhysicalPathGeneratedPolicyUseCase(UseCaseTest):
    path_policy = FS_PHYSICAL
    filename_policy = GENERATE_POLICY

class BasePathGeneratedPolicyUseCase(UseCaseTest):
    path_policy = FS_BASE
    filename_policy = GENERATE_POLICY


def test_suite():
    suite = unittest.TestSuite()
    files = (
        ('basic_operations.txt', BasicOperationsTest),
        ('delete.txt', FunctionalTest),
        ('rename.txt', FunctionalTest),
        ('functional.txt', FunctionalTest),
        ('usecases1.txt', UIDOriginalPolicyUseCase),
        ('usecases2.txt', UIDOriginalPolicyUseCase),
        ('usecases1.txt', UIDGeneratedPolicyUseCase),
        ('usecases2.txt', UIDGeneratedPolicyUseCase),
# XXX Those do not work yet.
#        ('usecases1.txt', RelativePathOriginalPolicyUseCase),
#        ('usecases2.txt', RelativePathOriginalPolicyUseCase),
#        ('usecases1.txt', RelativePathGeneratedPolicyUseCase),
#        ('usecases2.txt', RelativePathGeneratedPolicyUseCase),
#        ('usecases1.txt', PhysicalPathOriginalPolicyUseCase),
#        ('usecases2.txt', PhysicalPathOriginalPolicyUseCase),
#        ('usecases1.txt', PhysicalPathGeneratedPolicyUseCase),
#        ('usecases2.txt', PhysicalPathGeneratedPolicyUseCase),
#        ('usecases1.txt', BasePathOriginalPolicyUseCase),
#        ('usecases2.txt', BasePathOriginalPolicyUseCase),
#        ('usecases1.txt', BasePathGeneratedPolicyUseCase),
#        ('usecases2.txt', BasePathGeneratedPolicyUseCase),
        )
    def mksuite(fname, klass):
        return doctest.FunctionalDocFileSuite(
            fname,
            test_class=klass,
            package='Products.ATManagedFile.tests',
            globs=globals())
    for fname, klass in files:
        suite.addTest(mksuite(fname, klass))
    return suite

if __name__ == '__main__':
    framework(descriptions=0, verbosity=1)
