#!/usr/bin/env python2.3

# sync_verify.py | Copyright(C), 2004, Enfold Systems, LLC
#
# Enfold Systems, LLC
# 4500 Montrose Blvd., Suite D
# Houston, Texas 77006 USA
# p. +1 713.942.2377 | f. +1 832.201.8856
# www.enfoldsystems.com
# info@enfoldsystems.com
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# For a full copy of the license, please see the included 'LGPL.txt' file.
"""
$Id: repository.py 559 2004-12-03 02:11:23Z jlaplace $
"""

import os
import sys
import re
from optparse import OptionParser
import string
from normalizer.lang import normalize, registry, LangNormalizer

LANGS_DEFAULT = ['zope_invalid_punct', 'german', 'portuguese']
encoding = 'utf-8'
bad_id = re.compile(r'[^a-zA-Z0-9-_~,.$\(\)# ]')

ALLOWED_PUNCT = '-_~,.$()#'
zope_invalid_punct = LangNormalizer('zope_invalid_punct',
                                    [(c, '') for c in string.punctuation
                                     if c not in ALLOWED_PUNCT])
if not registry.is_registered(zope_invalid_punct.getLanguage()):
    registry.register(zope_invalid_punct.getLanguage(), zope_invalid_punct)

class Translator:

    def __init__(self, path, out, langs, verbose=False):
        self.path = path
        self.out = out
        self.verbose = False
        self.langs = langs

    def run(self):
        os.path.walk(self.path, self.node_action, ())

    def node_action(self, options, dirname, filelist):
        for fname in filelist:
            msgs = self.validate_name(fname)
            if msgs:
                self.log('%s' % os.path.join(dirname, fname))
                name = self.new_name(fname)
                msgs = self.validate_name(name)
                for msg in msgs:
                    self.log(msg)
                self.log('-> %s' % os.path.join(dirname, name))

    def new_name(self, name):
        name = normalize([name], self.langs)[0]
        if isinstance(name, unicode):
            name = name.encode(encoding)
        return name

    def log(self, msg, skip=True):
        if isinstance(msg, unicode):
            msg = msg.encode(encoding)
        self.out.write(msg)
        if skip:
            self.out.write('\n')

    def validate_name(self, name):
        msgs = []
        if bad_id.search(name) is not None:
            msgs.append('Contains invalid chars: %s - %s' %
                        (`name`, bad_id.findall(name)))
        elif name in ('.', '..'):
            msgs.append('Not an allowed name %s' % `name`)
        elif name.startswith('_'):
            msgs.append('Cannot start with an underscore: %s' % `name`)
        elif name.startswith('aq_'):
            msgs.append('Cannot start with an "aq_": %s' % `name`)
        elif name.endswith('__'):
            msgs.append('Cannot end with two underscores: %s' % `name`)
        return msgs

if __name__ == '__main__':
    parser = OptionParser()
    parser.add_option('-p', '--path',
                      dest='path',
                      help=('Path to be processed'))
    parser.add_option('-l', '--languages',
                      dest='languages',
                      help=('Languages to use for translating filenames to ids'))
    parser.add_option('-o', '--output',
                      dest='output',
                      help=('File where output should be saved'))
    parser.add_option('-v', '--verbose',
                      dest='verbose',
                      action='store_const',
                      const=True,
                      help=('Show warnings'))
    (options, args) = parser.parse_args()
    if not (options.path):
        parser.print_help()
        sys.exit(1)

    out = sys.stdout
    if options.output:
        out = open(os.path.normpath(options.output), 'wb')

    if not options.languages:
        langs = LANGS_DEFAULT
    else:
        langs = filter(None, [l.strip() for l in options.languages.split()])
    translator = Translator(options.path, out,
                            langs=langs, verbose=options.verbose)
    translator.run()
