# This work is based upon CMFExternalFile, which was developed by
# Alan Runyan with contributions by Andy McKay, Kiran Jonnalagadda
# and PloneExternalFile developed by Florian Geier.
#
# CMFManagedFile extends CMFExternalFile to include a concept that
# can be found in PloneExternalFile where uploaded documents may be
# routed to specific directories or file systems.  This is specified by the
# user at upload time through the selection of pre-defined "repositories"
# that the site administrator would define and establish.
#
# The merger of CMFExternalFile and PloneExternalFile has been further
# extended to include some aspects of managing the external files on the
# file systems.  These include the automatic deletion of the file system files
# for deleted documents of the class CMFManagedFile governed by customizable
# and user selectable deletion policies, and the automatic movement of files
# from one file system repository to another based upon an edit action
# that includes the selection of a repository other than the repository
# in which the file is currently stored.
#
"""
$Id: content.py 992 2005-02-10 02:55:45Z jlaplace $
"""

import os, os.path
import thread
import tempfile
from StringIO import StringIO
from Shared.DC.ZRDB.TM import TM
from Products.CMFManagedFile import config
from Products.ExternalFile.FileUtils import copy_file

class LStringIO(StringIO):

    def __len__(self):
        return self.len or 0

def chunked_read(wrapper, stream, amt, bs, total, pos, cb, res):
    while True:
        if amt < bs:
            bs = amt
        block = stream.read(bs)
        amt -= len(block)
        pos += len(block)
        if not block:
            break
        if cb is not None:
            cb(total, pos)
        res.write(block)
    return amt, pos

def file_or_stream_len(f):
    if hasattr(f, 'tell') and hasattr(f, 'seek'):
        pos = f.tell()
        f.seek(0, 2)
        size = f.tell()
        f.seek(pos)
        return size
    return len(f)

class CallbackStreamWrapper:

    def __init__(self, stream, cb, blocksize):
        self.stream = stream
        self.stream_len = file_or_stream_len(stream)
        self.cb = cb
        self.blocksize = blocksize
        self.pos = 0
        self.cache = LStringIO()

    def seek(self, pos):
        self.pos = pos
        self.cache.seek(pos)

    def read(self, amt=None):
        stream = self.stream
        cache = self.cache
        start = self.cache.pos
        res = LStringIO()
        # Once self.stream is consumed, if its
        # a socket, len will be set to 0, so we use
        # the cached value.
        # len(stream) is number of bytes left, not total.
        total = self.stream_len or len(cache)
        if amt is None:
            amt = total
        cb = self.cb
        bs = self.blocksize
        if cb is not None:
            cb.start()
        # First, read from the cache
        amt, self.pos = chunked_read(self, cache, amt, bs, total, self.pos,
                                     cb, res)
        # Then, if amount has not yet been reached, read from the stream
        if amt:
            amt, self.pos = chunked_read(self, stream, amt, bs, total,
                                             self.pos, cb, res)
        if start + len(res) > len(cache):
            # If more data was read from the stream, then
            # append this data to the cache.
            cache.seek(start)
            cache.write(res.getvalue())
        if self.pos == total and hasattr(cb, 'end'):
            cb.end()
        return res.getvalue()

    def __len__(self):
        return long(self.stream_len)

    def getvalue(self):
        return self.cache.getvalue()

class LimitedReadCallback:

    def __init__(self, max_bytes):
        self.reset()
        self.max_bytes = max_bytes

    def reset(self):
        pass

    start = exception = reset

    def __call__(self, total, position):
        if position > self.max_bytes:
            raise ValueError, ('Maximum file size limited '
                               'to %s bytes' % (self.max_bytes))
        return True

def limit_wrapper(stream, max_bytes):
    cb = LimitedReadCallback(max_bytes)
    return CallbackStreamWrapper(stream, cb, 2 << 14)

class FileManager(TM):

    def __init__(self):
        self.__files = {}

    def _tid(self):
        return thread.get_ident()

    def files(self):
        tid = self._tid()
        return self.__files.get(tid, ())

    def _reset(self):
        tid = self._tid()
        self.__files[tid] = []

    def _begin(self):
        assert not self.files(), 'Unprocessed files found in queue'

    def _abort(self):
        for src, dst in self.files():
            # Create parents if they don't exist
            parent = os.path.dirname(dst)
            if not os.path.exists(parent):
                os.makedirs(parent, mode=config.DIR_MODE)
            # Restore backup
            copy_file(src, dst)
            # Remove backup
            os.remove(src)
        self._reset()

    def _finish(self):
        # Remove backup, not needed anymore
        for src, dst in self.files():
            os.remove(src)
        # Empty files queue
        self._reset()

    def backup(self, fp):
        if not os.path.exists(fp):
            # Do nothing.
            return
        tid = self._tid()
        self._register()
        files = self.__files.setdefault(tid, [])
        # Temp filename for backup file
        nfp = tempfile.mktemp()
        # Copy file to new temporary filename
        copy_file(fp, nfp)
        # Add to queue
        files.append((nfp, fp))

file_manager = FileManager()
